{*************************************************************************}
{                                                                         }
{          Executor Component version 1.2 (SHAREWARE VERSION)             }
{                                                                         }
{  By using the non-visual Executor Component you are able to start any   }
{  Windows or DOS applications from within your Delphi application. The   }
{  Run command in the Windows Start menu acts almost the same, but you    }
{  can't use it in your own applications. With the Executor Component you }
{  are in control. If you want to, it stops your main process, until the  }
{  started program has finished.                                          }
{  You can open a file or folder or connect to a shared computer by       }
{  typing its path in the FileName property. If you want to have a DOS-   }
{  prompt assign the value 'command.com' to the property FileName.        }
{  Added ShellExecute support, like when you double click a file in your  }
{  explorer.                                                              }
{  Put "double quotes" around parameters with spaces in them.             }
{                                                                         }
{  Copyright (c) 1997,98 Erwin Denissen                                   }
{                                                                         }
{  You may use a Shareware Copy of this Software for an evaluation period }
{  of up to 30 days, in order to determine whether the Software meets     }
{  your needs before purchasing it.  Once the evaluation period ends, you }
{  agree to either purchase a Copy of the Software, or to stop using it   }
{  and remove it from your computer system.                               }
{                                                                         }
{  There are no known bugs at the moment, so please notify me when you    }
{  find one. Make sure you are using the latest version of this software. }
{  email: erwind@high-logic.com                                           }
{                                                                         }
{  A registered version of TExecutor can be ordered for US$20.            }
{  Order at the following site:                                           }
{  http://www.high-logic.com/register.html                                }
{                                                                         }
{*************************************************************************}
{                                                                         }
{  Published properties                                                   }
{  --------------------                                                   }
{  FileName    The name of the program                                    }
{  CommandLine The arguments                                              }
{  LaunchType  The File type you want to 'ShellExecute', for example .txt }
{  LaunchOperation the operation performed on the shell. Default is open. }
{  HideOwner   Set HideOwner to True, to hide                             }
{  Visibility  Well... try it.                                            }
{  Wait        the user must explicitly close the started process before  }
{              he/ she is able to access the parent process.              }
{                                                                         }
{*************************************************************************}
{                                                                         }
{ Version 1.2.1 17-11-1998                                                }
{   Added LaunchType property.                                            }
{   Added LaunchOperation property.                                       }
{                                                                         }
{ Version 1.2 15-11-1998                                                  }
{   Added CommandLine property.                                           }
{                                                                         }
{ Version 1.1 14-09-1998                                                  }
{   Fixed two problems, now works with Delphi 4                           }
{                                                                         }
{ Version 1.0 30-07-1997                                                  }
{   First release, works with Delphi 2 and 3                              }
{                                                                         }
{*************************************************************************}

unit Executor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, registry;

type

   TMyVisibility=
    ( HIDE,
      SHOWNORMAL,
      NORMAL,
      SHOWMINIMIZED,
      SHOWMAXIMIZED,
      MAXIMIZE,
      SHOWNOACTIVATE,
      SHOW,
      MINIMIZE,
      SHOWMINNOACTIVE,
      SHOWNA,
      RESTORE,
      SHOWDEFAULT,
      MAX
      );

  TExecutor = class(TComponent)
  private
    { Private declarations }
    FFileName: string;
    FCommandLine: string;
    FLaunchType : string;
    FLaunchOperation : string;
    FMyWait : Boolean;
    FMyHideOwner : Boolean;
    FMyVisibility : TMyVisibility;
    function GetFileName: string;
    function GetCommandLine: string;
    function GetLaunchType: string;
    function GetLaunchOperation: string;
    function GetWait:Boolean;
    procedure SetWait(Value : Boolean);
    function GetHideOwner:Boolean;
    procedure SetHideOwner(Value : Boolean);
    function GetVisibility:TMyVisibility;
    procedure SetVisibility(Value : TMyVisibility);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create (AOwner: TComponent); override;
    function Execute:DWORD;
    function GoOnline(URL:string):DWORD;
    procedure ShowRegisterBox;
  published
    { Published declarations }
    property Visibility : TMyVisibility read GetVisibility write SetVisibility default NORMAL;
    property HideOwner : Boolean read GetHideOwner write SetHideOwner default true;
    property Wait : Boolean read GetWait write SetWait default true;
    property FileName: string read GetFileName write FFileName;
    property LaunchType: string read GetLaunchType write FLaunchType;
    property LaunchOperation: string read GetLaunchOperation write FLaunchOperation;
    property CommandLine: string read GetCommandLine write FCommandLine;
  end;
procedure Register;

implementation

uses ShellApi, RegisterAbout;


procedure Register;
begin
  RegisterComponents('High-Logic', [TExecutor]);
end;

constructor TExecutor.Create (AOwner: TComponent);
begin
  inherited Create( AOwner);
  FMyVisibility:=NORMAL;
  FMyWait:=true;
  FMyHideOwner:=true;
end;

procedure TExecutor.ShowRegisterBox;
begin
  with TRegisterAboutBox.Create(Self) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

function TExecutor.GetWait:Boolean;
begin
  Result:=FMyWait;
end;

procedure TExecutor.SetWait(Value : Boolean);
begin
  FMyWait:=Value;
end;

function TExecutor.GetHideOwner:Boolean;
begin
  Result:=FMyHideOwner;
end;

procedure TExecutor.SetHideOwner(Value : Boolean);
begin
  FMyHideOwner:=Value;
end;

function TExecutor.GetVisibility:TMyVisibility;
begin
  Result:=FMyVisibility;
end;

procedure TExecutor.SetVisibility(Value : TMyVisibility);
begin
  FMyVisibility:=Value;
end;

function TExecutor.GetFileName: string;
begin
  Result := FFileName;
end;

function TExecutor.GetCommandLine: string;
begin
  Result := FCommandLine;
end;

function TExecutor.GetLaunchType: string;
begin
  Result := FLaunchType;
end;

function TExecutor.GetLaunchOperation: string;
begin
  Result := FLaunchOperation;
end;

function TExecutor.Execute:DWORD;
var
  buffer, zAppName, zCommandLine, Hulp : array[0..512-1] of char;
  zCurDir  : array[0..256-1] of char;
  WorkDir  : string;
  StartUpInfo :TStartUpInfo;
  ProcessInfo : TProcessInformation;
  myReg : TRegistry;
  FileNameClass, NewAppName, AppName, TheLaunchOperation : string;
  i : integer;
begin
  if LaunchOperation=''
  then TheLaunchOperation:='open'
  else TheLaunchOperation:=LaunchOperation;
  Result:=0;
  StrPCopy(zAppName, FileName);
  StrPCopy(zCommandLine, ' '+CommandLine);
  GetDir(0,WorkDir);
  StrPCopy(zCurDir,WorkDir);
  FillChar(StartUpInfo,sizeof(StartUpInfo),#0);
  StartUpInfo.cb:=sizeof(StartUpInfo);
  StartUpInfo.dwFlags:=STARTF_USESHOWWINDOW;
  case FMyVisibility of
    HIDE             : StartUpInfo.wShowWindow:=SW_HIDE;
    SHOWNORMAL       : StartUpInfo.wShowWindow:=SW_SHOWNORMAL;
    NORMAL           : StartUpInfo.wShowWindow:=SW_NORMAL;
    SHOWMINIMIZED    : StartUpInfo.wShowWindow:=SW_SHOWMINIMIZED;
    SHOWMAXIMIZED    : StartUpInfo.wShowWindow:=SW_SHOWMAXIMIZED;
    MAXIMIZE         : StartUpInfo.wShowWindow:=SW_MAXIMIZE;
    SHOWNOACTIVATE   : StartUpInfo.wShowWindow:=SW_SHOWNOACTIVATE;
    SHOW             : StartUpInfo.wShowWindow:=SW_SHOW;
    MINIMIZE         : StartUpInfo.wShowWindow:=SW_MINIMIZE;
    SHOWMINNOACTIVE  : StartUpInfo.wShowWindow:=SW_SHOWMINNOACTIVE;
    SHOWNA           : StartUpInfo.wShowWindow:=SW_SHOWNA;
    RESTORE          : StartUpInfo.wShowWindow:=SW_RESTORE;
    SHOWDEFAULT      : StartUpInfo.wShowWindow:=SW_SHOWDEFAULT;
    MAX              : StartUpInfo.wShowWindow:=SW_MAX;
  end;
  FindExecutable(zAppName, zCurDir, Buffer);
  if FileName='' then  
  begin
    if LaunchType='' then
    begin // use command.com??? specified in Buffer
      if not CreateProcess(Buffer, nil, nil, nil, false,
                      CREATE_NEW_CONSOLE or NORMAL_PRIORITY_CLASS, nil, nil,
                      StartUpInfo, ProcessInfo)
      then Result:=1;
    end
    else
    begin // Shell find exe in registry...
      myReg:=TRegistry.Create;
      myReg.RootKey:=HKEY_CLASSES_ROOT;
      myReg.OpenKey('\'+LaunchType, false);
      FileNameClass:=myReg.ReadString('');
      if FileNameClass<>'' then
      begin
        myReg.OpenKey('\'+FileNameClass+'\shell\'+TheLaunchOperation+'\command', false);
        AppName:=myReg.ReadString('');
      end;
      myReg.Free;
      if AppName<>'' then
      begin
        i:=1;
        while (AppName[i]<>'%') and (i<length(AppName)) do inc(i);
        if AppName[i]='%' then
        begin
          NewAppName:=Copy(AppName, 1, i-3);
          StrPCopy(zAppName, NewAppName);
          StrPCopy(zCommandLine, ' '+CommandLine);
          if not CreateProcess(zAppName , zCommandLine, nil, nil, false,
                          CREATE_NEW_CONSOLE or NORMAL_PRIORITY_CLASS, nil, nil,
                          StartUpInfo, ProcessInfo)
          then
          begin
            NewAppName:=Copy(AppName, 1, i-2)+' '+CommandLine;
            StrPCopy(zAppName, NewAppName);
            if not CreateProcess(zAppName , nil, nil, nil, false,
                            CREATE_NEW_CONSOLE or NORMAL_PRIORITY_CLASS, nil, nil,
                            StartUpInfo, ProcessInfo)
            then
              if not CreateProcess(nil , zAppName, nil, nil, false,
                              CREATE_NEW_CONSOLE or NORMAL_PRIORITY_CLASS, nil, nil,
                              StartUpInfo, ProcessInfo)
              then Result:=1;
          end;
          exit;
        end;
        if (AppName[1]='"') and (length(AppName)>1) then
        begin
          i:=2;
          while (AppName[i]<>'"') and (i<length(AppName)) do inc(i);
          NewAppName:=Copy(AppName, 2, i-2);
          if length(AppName)>i then
          begin
            inc(i);
            while (AppName[i]=' ') and (i<length(AppName)) do inc(i);
            if length(AppName)>i
            then CommandLine:=Copy(AppName, i, length(AppName)-i+1)+' '+CommandLine;
          end;
        end;
      end;
      StrPCopy(zAppName, NewAppName);
      StrPCopy(zCommandLine, CommandLine);
      if not CreateProcess(zAppName , zCommandLine, nil, nil, false,
                      CREATE_NEW_CONSOLE or NORMAL_PRIORITY_CLASS, nil, nil,
                      StartUpInfo, ProcessInfo)
      then Result:=1;
    end;
  end
  else
  begin
    if LaunchType='' then // use FileName
    begin
      if CommandLine<>'' then
      begin
        if not CreateProcess(zAppName , zCommandLine, nil, nil, false,
                        CREATE_NEW_CONSOLE or NORMAL_PRIORITY_CLASS, nil, nil,
                        StartUpInfo, ProcessInfo)
        then
          if not CreateProcess(Buffer , zCommandLine, nil, nil, false,
                          CREATE_NEW_CONSOLE or NORMAL_PRIORITY_CLASS, nil, nil,
                          StartUpInfo, ProcessInfo)
          then
          begin
            StrPCopy(Hulp, ' ');
            StrCat(Buffer, Hulp);
            StrCat(Buffer, zCommandLine);
            if not CreateProcess(nil, Buffer, nil, nil, false,
                            CREATE_NEW_CONSOLE or NORMAL_PRIORITY_CLASS, nil, nil,
                            StartUpInfo, ProcessInfo)
            then Result:=1;
          end;
      end
      else
      begin
        if not CreateProcess(zAppName , nil, nil, nil, false,
                        CREATE_NEW_CONSOLE or NORMAL_PRIORITY_CLASS, nil, nil,
                        StartUpInfo, ProcessInfo)
        then
        begin
          if not CreateProcess(nil , zAppName, nil, nil, false,
                          CREATE_NEW_CONSOLE or NORMAL_PRIORITY_CLASS, nil, nil,
                          StartUpInfo, ProcessInfo)
          then
          begin
            StrPCopy(Hulp, ' ');
            StrCat(Buffer, Hulp);
            StrCat(Buffer, zAppName);
            if not CreateProcess(nil, Buffer, nil, nil, false,
                            CREATE_NEW_CONSOLE or NORMAL_PRIORITY_CLASS, nil, nil,
                            StartUpInfo, ProcessInfo)
            then Result:=1;
          end;
        end;
      end;
    end
    else
    begin
    end;
  end;
  if Result=0 then // everything is ok
  begin
    if HideOwner
    then with Owner as TForm do hide;
    if FMyWait
    then WaitForSingleObject(ProcessInfo.hProcess, INFINITE);
    GetExitCodeProcess(ProcessInfo.hProcess, Result);
    CloseHandle(ProcessInfo.hProcess);
    CloseHandle(ProcessInfo.hThread);
    if HideOwner
    then with Owner as TForm do show;
  end;
end;

function TExecutor.GoOnline(URL:string):DWORD;
begin
  Visibility:=MAXIMIZE;
  Wait:=false;
  HideOwner:=false;
  LaunchType:='.html';
  LaunchOperation:='open';
  CommandLine:='http://www.high-logic.com/cool.html';
  result:=Execute;
end;

end.
