{*************************************************************************}
{                                                                         }
{         UndoAndRedo Component version 1.2 (SHAREWARE VERSION)           }
{                                                                         }
{  By using the UndoAndRedo Component you are able to control the undo,   }
{  repeat and redo functionality in an easy way. This proffesionality is  }
{  provided in one component. It's easy to use and really cool if you     }
{  understand the internal functionality.                                 }
{                                                                         }
{  Copyright (c) 1997, 1998 High-Logic [Erwin Denissen]                   }
{                                                                         }
{  You may use a Shareware Copy of this Software for an evaluation period }
{  of up to 99 days, in order to determine whether the Software meets     }
{  your needs before purchasing it.  Once the evaluation period ends, you }
{  agree to either purchase a Copy of the Software, or to stop using it   }
{  and remove it from your computer system.                               }
{                                                                         }
{  There are no known bugs at the moment, so please notify me when you    }
{  find one. Make sure you are using the latest version of this software. }
{  email: erwind@high-logic.com                                           }
{                                                                         }
{  A registered version of TUndoAndRedo can be ordered for US$40.         }
{  order at the following site: http://www.high-logic.com/index.html      }
{                                                                         }
{*************************************************************************}
{                                                                         }
{  Available methods                                                      }
{  -----------------                                                      }
{                                                                         }
{*************************************************************************}
{                                                                         }
{ Version 1.2 19-10-1998                                                  }
{  First official release                                                 }
{                                                                         }
{ Version 1.103 14-10-1998                                                }
{ Added 4 Methods                                                         }
{  procedure LoadFromStream(Stream: TStream);                             }
{  procedure SaveToStream(Stream: TStream);                               }
{  procedure LoadFromFile(const FileName: string);                        }
{  procedure SaveToFile(const FileName: string);                          }
{                                                                         }
{ Version 1.102 10-10-1998                                                }
{   Added Group support and the Active property                           }
{                                                                         }
{ Version 1.1 14-09-1998                                                  }
{   Fixed two problems, now works with Delphi 4                           }
{                                                                         }
{ Version 1.0 30-07-1997                                                  }
{   First release, works with Delphi 2 and 3                              }
{                                                                         }
{*************************************************************************}
{                                                                         }
{ More things to do:                                                      }
{                                                                         }
{ * More debug messages and other debug options                           }
{ * Better Documentation in the code but also in some readme file         }
{ * Maybe change the MenuItem Support, because Delphi 4.0 has a new       }
{   action component.                                                     }
{ * Recalculate the MemoryAllocationSize at some points                   }
{                                                                         }
{*************************************************************************}

unit UndoAndRedo;

interface

uses SysUtils, Dialogs, Classes, Menus;

type

  TFunctionVariable =function(TheData1, TheData2, TheData3: pointer):pointer of object;

  TPointerRecord =
  record
    procedureVariable : TFunctionVariable;
    procedureData1 : pointer;
    procedureData2 : pointer;
    procedureData3 : pointer;
    lengte1, lengte2, lengte3 : longint;
    Number : integer;
  end;
  TPointerArrayRecord = array [1..1] of TPointerRecord;
  PPointerArray = ^TPointerArrayRecord;

  TName = string[25];
  TNameArray = array [1..1] of TName;
  PNameArray = ^TNameArray;

TUndoAndRedo = class(TComponent)

private
  FActive : Boolean;
  DebugMode : Boolean;
  MIUndo : TMenuItem;
  MIRedo : TMenuItem;
  MIRepeat : TMenuItem;
  MIAllowGroupUndo : Boolean;
  PossibleGroupUndo : Boolean;
  NumberOfRepeatPointers, NumberOfNames, TheTempNumber, CurrentPointer, LastPointer, NumberOfPointers : integer;
  TheRepeatPointers, TheTempPointers, ThePointers : PPointerArray;
  TheNames : PNameArray;
  InRepeatMode, Started, InUndoMode, InRedoMode, Updating : boolean;
  procedure SetActiveProperty(Value : Boolean);
  procedure FreeRepeatPointers;
  procedure SetMenuItems;
  procedure ShowMessage(TheMessage: string);
public
//  constructor Create;
  MemoryAllocationSize : longint;
  constructor Create (AOwner: TComponent); override;
  destructor Destroy; override;
  procedure Add(TheFunction : TFunctionVariable; TheData1, TheData2, TheData3: pointer; TheSize1, TheSize2, TheSize3 : integer );
  procedure AddRepeat(TheFunction : TFunctionVariable; TheData1, TheData2, TheData3: pointer; TheSize1, TheSize2, TheSize3 : integer );
  procedure AddToPrevious(TheFunction : TFunctionVariable; TheData1, TheData2, TheData3: pointer; TheSize1, TheSize2, TheSize3 : integer );
  function Undo:boolean;
  function Redo:boolean;
  function DoRepeat:boolean;
  function CanUndo:boolean;
  function CanRedo:boolean;
  function CanRepeat:boolean;
  procedure UndoBeginUpdate(TheName : TName);
  procedure UndoEndUpdate;
  function IsUpdating:boolean;
  function GetUndoName: TName;
  function GetRedoName: TName;
  function GetUndoItems: TStringList;
  function GetRedoItems: TStringList;
  function GetRepeatName: TName;
  procedure Reset;
  procedure ShowRegisterBox;
  procedure LoadFromStream(Stream: TStream);
  procedure SaveToStream(Stream: TStream);
  procedure LoadFromFile(const FileName: string);
  procedure SaveToFile(const FileName: string);
published
  property Active:Boolean read FActive write SetActiveProperty;
  property Debugging:Boolean read DebugMode write DebugMode;
  property MenuItemUndo:TMenuItem read MIUndo write MIUndo;
  property MenuItemRedo:TMenuItem read MIRedo write MIRedo;
  property MenuItemRepeat:TMenuItem read MIRepeat write MIRepeat;
  property AllowGroupUndo:Boolean read MIAllowGroupUndo write MIAllowGroupUndo;
  {}
end;

procedure Register;

implementation

uses RegisterAbout;

type
  TUndoAndRedofileHeader = record
    Version : integer;
    VersionInfo : array[1..10] of char;
    NumberOfPoints,
    CurrentPointer,
    LastPointer,
    NumberOfNames : integer;
    Reserved : integer;
  end;
  TUndoAndRedoNameRecord = record
    Name : TName;
  end;
  TUndoAndRedoPointerRecord = record
    Size1, Size2, Size3 : integer;
    procedureVariable : TFunctionVariable;
    Number : integer;
  end;

{$R-}

procedure Register;
begin
  RegisterComponents('High-Logic', [TUndoAndRedo]);
end;

//constructor TUndoAndRedo.Create;
constructor TUndoAndRedo.Create (AOwner: TComponent);
begin
  inherited Create( AOwner);
//  inherited Create;
  Active:=True;
  Updating:=false;
  InUndoMode:=false;
  InRedoMode:=false;
  NumberOfPointers:=0;
  CurrentPointer:=0;
  LastPointer:=0;
  ThePointers:=nil;
  NumberOfNames:=0;
  TheNames:=nil;
  TheTempPointers:=nil;
  NumberOfRepeatPointers:=0;
  TheRepeatPointers:=nil;
  InRepeatMode:=false;
  SetMenuItems;
  AllowGroupUndo:=True;
  PossibleGroupUndo:=False;
//  ShowNagMessage
  DebugMode:=True;
  MemoryAllocationSize:=0;
end;

destructor TUndoAndRedo.Destroy;
var
  t : integer;
begin
  FreeMem(TheNames);
  if NumberOfPointers>0
  then for t:=1 to NumberOfPointers do
  begin
    FreeMem(ThePointers[t].procedureData1);
    FreeMem(ThePointers[t].procedureData2);
    FreeMem(ThePointers[t].procedureData3);
  end;
  FreeMem(ThePointers);
  NumberOfRepeatPointers:=0;
  CurrentPointer:=0;
  LastPointer:=0;
{  MenuItemUndo.Enabled:=False;
  MenuItemUndo.Caption:='&Undo';
  MenuItemRedo.Enabled:=False;
  MenuItemRedo.Caption:='&Redo';
  MenuItemRepeat.Enabled:=False;
  MenuItemRepeat.Caption:='R&epeat';}
  inherited Destroy;
end;

procedure TUndoAndRedo.SetActiveProperty(Value : Boolean);
begin
  if Value<>FActive then
  begin
    FActive:=Value;
    SetMenuItems;
  end;
end;

procedure TUndoAndRedo.ShowRegisterBox;
begin
  with TRegisterAboutBox.Create(Self) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

procedure TUndoAndRedo.ShowMessage(TheMessage: string);
begin
  if DebugMode then
  begin
    MessageDlg(TheMessage, mtError, [mbOK], 0);
  end;
end;

procedure TUndoAndRedo.Reset;
var
  t : integer;
begin
  if not (Updating or InUndoMode or InRedoMode) then
  begin
    FreeMem(TheNames);
    TheNames:=nil;
    if NumberOfPointers>0
    then for t:=1 to NumberOfPointers do
    begin
      FreeMem(ThePointers[t].procedureData1);
      FreeMem(ThePointers[t].procedureData2);
      FreeMem(ThePointers[t].procedureData3);
    end;
    FreeMem(ThePointers);
    ThePointers:=nil;
    Updating:=false;
    InUndoMode:=false;
    InRedoMode:=false;
    NumberOfPointers:=0;
    CurrentPointer:=0;
    LastPointer:=0;
    NumberOfNames:=0;
    NumberOfRepeatPointers:=0;
    MemoryAllocationSize:=0;
    SetMenuItems;
  end;
end;

function TUndoAndRedo.GetRepeatName: TName;
begin
  if CanRepeat
  then result:=TheNames[NumberOfNames]
  else result:='';
end;

function TUndoAndRedo.GetUndoName: TName;
begin
  if CanUndo
  then result:=TheNames[CurrentPointer]
  else result:='';
end;

function TUndoAndRedo.GetRedoName: TName;
begin
  if CanRedo
  then result:=TheNames[CurrentPointer+1]
  else result:='';
end;

function TUndoAndRedo.GetUndoItems: TStringList;
var
  TheItems : TStringList;
  i : integer;
begin
  TheItems:=TStringList.Create;
  if CanUndo then
  begin
    for i:=CurrentPointer downto 1 do
    TheItems.Add(TheNames[i]);
  end
  else
  begin
    TheItems.Clear;
  end;
  result:=TheItems;
end;


function TUndoAndRedo.GetRedoItems: TStringList;
var
  TheItems : TStringList;
  i : integer;
begin
  TheItems:=TStringList.Create;
  if CanRedo then
  begin
    for i:=CurrentPointer+1 to LastPointer do
    TheItems.Add(TheNames[i]);
  end
  else
  begin
    TheItems.Clear;
  end;
  result:=TheItems;
{  if CanRedo
  then result:=TheNames[CurrentPointer+1]
  else result:='';}
end;


procedure TUndoAndRedo.AddToPrevious(TheFunction : TFunctionVariable; TheData1, TheData2, TheData3: pointer; TheSize1, TheSize2, TheSize3 : integer );
begin
  if Active then
  begin
    if Updating and (not Started) then
    begin
      inc(NumberOfPointers);
      ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
      with ThePointers[NumberOfPointers] do
      begin
        procedureVariable:=TheFunction;
        procedureData1:=AllocMem(TheSize1);
        lengte1:=TheSize1;
        move(TheData1^, procedureData1^, TheSize1);
        procedureData2:=AllocMem(TheSize2);
        lengte2:=TheSize2;
        move(TheData2^, procedureData2^, TheSize2);
        procedureData3:=AllocMem(TheSize3);
        lengte3:=TheSize3;
        move(TheData3^, procedureData3^, TheSize3);
        Number:=CurrentPointer-1;
        MemoryAllocationSize:=MemoryAllocationSize+lengte1+lengte2+lengte3+3*SizeOf(Longint)+SizeOf(Pointer);
      end;
    end;
  end;
end;

procedure TUndoAndRedo.Add(TheFunction : TFunctionVariable; TheData1, TheData2, TheData3: pointer; TheSize1, TheSize2, TheSize3 : integer );
begin
  if Active then
  begin
    if InRedoMode or InUndoMode or InRepeatMode then
    begin
      inc(TheTempNumber);
      ReAllocMem(TheTempPointers, TheTempNumber*SizeOf(TPointerRecord));
      with TheTempPointers[TheTempNumber] do
      begin
        procedureVariable:=TheFunction;
        procedureData1:=AllocMem(TheSize1);
        lengte1:=TheSize1;
        move(TheData1^, procedureData1^, TheSize1);
        procedureData2:=AllocMem(TheSize2);
        lengte2:=TheSize2;
        move(TheData2^, procedureData2^, TheSize2);
        procedureData3:=AllocMem(TheSize3);
        lengte3:=TheSize3;
        move(TheData3^, procedureData3^, TheSize3);
        if InUndoMode
        then Number:=CurrentPointer
        else Number:=CurrentPointer+1;
        MemoryAllocationSize:=MemoryAllocationSize+lengte1+lengte2+lengte3+3*SizeOf(Longint)+SizeOf(Pointer);
      end;
    end
    else
    if Updating then
    begin
      Started:=true;
      inc(NumberOfPointers);
      ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
      with ThePointers[NumberOfPointers] do
      begin
        procedureVariable:=TheFunction;
        procedureData1:=AllocMem(TheSize1);
        lengte1:=TheSize1;
        move(TheData1^, procedureData1^, TheSize1);
        procedureData2:=AllocMem(TheSize2);
        lengte2:=TheSize2;
        move(TheData2^, procedureData2^, TheSize2);
        procedureData3:=AllocMem(TheSize3);
        lengte3:=TheSize3;
        move(TheData3^, procedureData3^, TheSize3);
        Number:=CurrentPointer;
        MemoryAllocationSize:=MemoryAllocationSize+lengte1+lengte2+lengte3+3*SizeOf(Longint)+SizeOf(Pointer);
      end;
    end;
    SetMenuItems;
  end;
end;

procedure TUndoAndRedo.AddRepeat(TheFunction : TFunctionVariable; TheData1, TheData2, TheData3: pointer; TheSize1, TheSize2, TheSize3 : integer );
begin
  if Active then
  begin
    if Updating then
    begin
      inc(NumberOfRepeatPointers);
      ReAllocMem(TheRepeatPointers, NumberOfRepeatPointers*SizeOf(TPointerRecord));
      with TheRepeatPointers[NumberOfRepeatPointers] do
      begin
        procedureVariable:=TheFunction;
        procedureData1:=AllocMem(TheSize1);
        lengte1:=TheSize1;
        move(TheData1^, procedureData1^, TheSize1);
        procedureData2:=AllocMem(TheSize2);
        lengte2:=TheSize2;
        move(TheData2^, procedureData2^, TheSize2);
        procedureData3:=AllocMem(TheSize3);
        lengte3:=TheSize3;
        move(TheData3^, procedureData3^, TheSize3);
        Number:=CurrentPointer;
        MemoryAllocationSize:=MemoryAllocationSize+lengte1+lengte2+lengte3+3*SizeOf(Longint)+SizeOf(Pointer);
      end;
    end;
    SetMenuItems;
  end;
end;

procedure TUndoAndRedo.FreeRepeatPointers;
var
  t : integer;
begin
  if NumberOfRepeatPointers>0 then
  begin
    for t:=1 to NumberOfRepeatPointers do with TheRepeatPointers[t] do
    begin
      FreeMem(procedureData1);
      FreeMem(procedureData2);
      FreeMem(procedureData3);
      MemoryAllocationSize:=MemoryAllocationSize-lengte1-lengte2-lengte3-3*SizeOf(Longint)-SizeOf(Pointer);
    end;
    FreeMem(TheRepeatPointers);
    TheRepeatPointers:=nil;
    NumberOfRepeatPointers:=0;
  end;
end;


function TUndoAndRedo.Undo:boolean;
var
  LastOfThisNumber, TheTotalOfThisNumber, u, i, t : integer;
begin
  result:=false;
  if Active then
  begin
    if Updating
    then ShowMessage('Can''t Undo, in updating mode!')
    else if CanUndo then
    begin
      InUndoMode:=true;
      FreeRepeatPointers;
      TheTempNumber:=0;
      TheTotalOfThisNumber:=0;
      t:=NumberOfPointers;
      while ThePointers[t].Number>CurrentPointer do dec(t);
      result:=true;
      repeat
        inc(TheTotalOfThisNumber);
        with ThePointers[t] do
        begin
          procedureVariable(procedureData1, procedureData2, procedureData3);
          FreeMem(procedureData1);
          procedureData1:=nil;
          FreeMem(procedureData2);
          procedureData2:=nil;
          FreeMem(procedureData3);
          procedureData3:=nil;
          MemoryAllocationSize:=MemoryAllocationSize-lengte1-lengte2-lengte3-3*SizeOf(Longint)-SizeOf(Pointer);
        end;
        dec(t);
      until (t=0) or (ThePointers[t].Number<CurrentPointer);
  // Voeg TheTempPointer in ThePointer
      if TheTempNumber>0 then
      begin
        if TheTempNumber<>TheTotalOfThisNumber then
        begin
          LastOfThisNumber:=t+TheTotalOfThisNumber;
          NumberOfPointers:=NumberOfPointers+TheTempNumber-TheTotalOfThisNumber;
          if TheTempNumber>TheTotalOfThisNumber then
          begin // uitbreiden
            ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
            for u:=NumberOfPointers-TheTempNumber+TheTotalOfThisNumber downto LastOfThisNumber
            do ThePointers[u+TheTempNumber-TheTotalOfThisNumber]:=ThePointers[u];
          end
          else
          begin // inkrimpen
            if LastOfThisNumber<NumberOfPointers-TheTempNumber+TheTotalOfThisNumber then
            for u:= LastOfThisNumber+1 to NumberOfPointers-TheTempNumber+TheTotalOfThisNumber
            do ThePointers[u+TheTempNumber-TheTotalOfThisNumber]:=ThePointers[u];
            ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
          end
        end;
        i:=TheTempNumber;
        repeat
          inc(t);
          ThePointers[t]:=TheTempPointers[i];
          dec(i);
        until i=0;
        FreeMem(TheTempPointers);
        TheTempPointers:=nil;
      end;
      dec(CurrentPointer);
      InUndoMode:=false;
    end;
    SetMenuItems;
    PossibleGroupUndo:=False;
  end;
end;

function TUndoAndRedo.DoRepeat:boolean;
var
  {LastOfThisNumber, TheTotalOfThisNumber, u, }i, t : integer;
begin
  result:=false;
  if Active then
  begin
    if Updating
    then ShowMessage('Can''t Repeat, in updating mode!')
    else
    begin
      if NumberOfRepeatPointers>0 then
      begin
        TheTempNumber:=0;
        InRepeatMode:=true;
        result:=true;
        for t:=1 to NumberOfRepeatPointers do with TheRepeatPointers[t] do
        begin
          procedureVariable(procedureData1, procedureData2, procedureData3);
  {        FreeMem(procedureData1);
          procedureData1:=nil;
          FreeMem(procedureData2);
          procedureData2:=nil;
          FreeMem(procedureData3);
          procedureData3:=nil;}
        end;
        InRepeatMode:=false;
      end;
  // Voeg TheTempPointer in ThePointer
      if TheTempNumber>0 then
      begin
  //      if TheTempNumber<>TheTotalOfThisNumber then
        begin
  //        LastOfThisNumber:=t+TheTotalOfThisNumber;
          CurrentPointer:=ThePointers[NumberOfPointers].Number+1;
          NumberOfPointers:=NumberOfPointers+TheTempNumber{-TheTotalOfThisNumber};
  //        if TheTempNumber>TheTotalOfThisNumber then
          begin // uitbreiden
            ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
  (*          for u:=NumberOfPointers{-TheTempNumber{+TheTotalOfThisNumber} downto NumberOfPointers-TheTempNumber{LastOfThisNumber}
            do ThePointers[u+TheTempNumber{-TheTotalOfThisNumber}]:=ThePointers[u];*)
          end
  //        else
  {        begin // inkrimpen
            if LastOfThisNumber<NumberOfPointers-TheTempNumber+TheTotalOfThisNumber then
            for u:= LastOfThisNumber+1 to NumberOfPointers-TheTempNumber+TheTotalOfThisNumber
            do ThePointers[u+TheTempNumber-TheTotalOfThisNumber]:=ThePointers[u];
            ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
          end}
        end;
        t:=NumberOfPointers-TheTempNumber;
        i:=TheTempNumber;
        repeat
          inc(t);
          ThePointers[t]:=TheTempPointers[i];
          dec(i);
        until i=0;
        FreeMem(TheTempPointers);
        TheTempPointers:=nil;
        TheTempNumber:=0;

  //    inc(CurrentPointer);
      if NumberOfNames<CurrentPointer
      then ReAllocMem(TheNames, CurrentPointer*SizeOf(TName));
      NumberOfNames:=CurrentPointer;
      TheNames[CurrentPointer]:=TheNames[CurrentPointer-1];
      LastPointer:=CurrentPointer;

      end;
    end;
    SetMenuItems;
//    PossibleGroupUndo:=False;
  end;
end;


function TUndoAndRedo.Redo:boolean;
var
  LastOfThisNumber, TheTotalOfThisNumber, u, i, t : integer;
begin
  result:=false;
  if Active then
  begin
    if Updating
    then ShowMessage('Can''t Redo, in updating mode!')
    else if CanRedo then
    begin
      InRedoMode:=true;
      FreeRepeatPointers;
      TheTempNumber:=0;
      TheTotalOfThisNumber:=0;
      t:=1;
      while ThePointers[t].Number<=CurrentPointer do inc(t);
      result:=true;
      repeat
        inc(TheTotalOfThisNumber);
        with ThePointers[t] do
        begin
          procedureVariable(procedureData1, procedureData2, procedureData3);
          FreeMem(procedureData1);
          procedureData1:=nil;
          FreeMem(procedureData2);
          procedureData2:=nil;
          FreeMem(procedureData3);
          procedureData3:=nil;
          MemoryAllocationSize:=MemoryAllocationSize-lengte1-lengte2-lengte3-3*SizeOf(Longint)-SizeOf(Pointer);
        end;
        inc(t);
      until (t>NumberOfPointers) or (ThePointers[t].Number>CurrentPointer+1);
  // Voeg TheTempPointer in ThePointer
      t:=t-TheTotalOfThisNumber-1;
      if TheTempNumber>0 then
      begin
        if TheTempNumber<>TheTotalOfThisNumber then
        begin
          LastOfThisNumber:=t+TheTotalOfThisNumber;
          NumberOfPointers:=NumberOfPointers+TheTempNumber-TheTotalOfThisNumber;
          if TheTempNumber>TheTotalOfThisNumber then
          begin // uitbreiden
            ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
            if LastOfThisNumber<NumberOfPointers-TheTempNumber+TheTotalOfThisNumber then
            for u:=NumberOfPointers-TheTempNumber+TheTotalOfThisNumber downto LastOfThisNumber+1
            do ThePointers[u+TheTempNumber-TheTotalOfThisNumber]:=ThePointers[u];
          end
          else
          begin // inkrimpen
            if LastOfThisNumber<NumberOfPointers-TheTempNumber+TheTotalOfThisNumber then
            for u:= LastOfThisNumber+1 to NumberOfPointers-TheTempNumber+TheTotalOfThisNumber
            do ThePointers[u+TheTempNumber-TheTotalOfThisNumber]:=ThePointers[u];
            ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
          end
        end;
        i:=0;
        repeat
          inc(i);
          inc(t);
          ThePointers[t]:=TheTempPointers[i];
        until i=TheTempNumber;
        FreeMem(TheTempPointers);
        TheTempPointers:=nil;
      end;
      inc(CurrentPointer);
      InRedoMode:=false;
    end;
    SetMenuItems;
    PossibleGroupUndo:=False;
  end;
end;

function TUndoAndRedo.CanRepeat:boolean;
begin
  result:=(NumberOfRepeatPointers>0) and (not Updating) and (Active);
end;

function TUndoAndRedo.CanUndo:boolean;
begin
  result:=(CurrentPointer<>0) and (not Updating) and (Active);
end;

function TUndoAndRedo.CanRedo:boolean;
begin
  result:=(LastPointer<>CurrentPointer) and (not Updating) and (Active);
end;

procedure TUndoAndRedo.UndoBeginUpdate(TheName : TName);
var
  i, t : integer;
begin
  if Active then
  begin
    if Updating
    then ShowMessage('Already updating the undo option!')
    else
    if InRedoMode or InUndoMode or InRepeatMode
    then ShowMessage('Already in some other undo mode!')
    else
    begin
      SetMenuItems;
      FreeRepeatPointers;
      if (CurrentPointer<>LastPointer) and (NumberOfPointers>0) then
      begin
        t:=1;
        while ThePointers[t].Number<=CurrentPointer do inc(t);
        for i:=t to NumberOfPointers do with ThePointers[i] do
        begin
          FreeMem(procedureData1);
          FreeMem(procedureData2);
          FreeMem(procedureData3);
          MemoryAllocationSize:=MemoryAllocationSize-lengte1-lengte2-lengte3-3*SizeOf(Longint)-SizeOf(Pointer);
        end;
        NumberOfPointers:=t-1;
        ReAllocMem(ThePointers, NumberOfPointers*SizeOf(TPointerRecord));
      end;
      if (CurrentPointer>0) and (AllowGroupUndo) then
      begin
        if (TheNames[CurrentPointer]=TheName) and (PossibleGroupUndo)
        then
        begin
        end
        else inc(CurrentPointer);
      end
      else inc(CurrentPointer);
      if NumberOfNames<CurrentPointer
      then ReAllocMem(TheNames, CurrentPointer*SizeOf(TName));
      NumberOfNames:=CurrentPointer;
      TheNames[CurrentPointer]:=TheName;
      LastPointer:=CurrentPointer;
      Updating:=true;
      Started:=false;
    end;
  end;
end;

procedure TUndoAndRedo.UndoEndUpdate;
begin
  if Active then
  begin
    if NumberOfPointers=0 then
    begin
      CurrentPointer:=0;
      LastPointer:=0;
      FreeRepeatPointers;
    end
    else
      if ThePointers[NumberOfPointers].Number>CurrentPointer
      then
      begin
        LastPointer:=CurrentPointer;
        FreeRepeatPointers;
      end
      else
      begin
        CurrentPointer:=ThePointers[NumberOfPointers].Number;
        LastPointer:=ThePointers[NumberOfPointers].Number;
      end;
    Updating:=false;
    SetMenuItems;
    PossibleGroupUndo:=True;
  end;
end;

function TUndoAndRedo.IsUpdating:boolean;
begin
  result:=Updating;
end;

procedure TUndoAndRedo.SetMenuItems;
begin
  if Assigned(MenuItemUndo) then
  begin
    MenuItemUndo.Enabled:=CanUndo;
    MenuItemUndo.Caption:='&Undo';
    if CanUndo
    then MenuItemUndo.Caption:=MenuItemUndo.Caption+' '+GetUndoName;
  end;
  if Assigned(MenuItemRedo) then
  begin
    MenuItemRedo.Enabled:=CanRedo;
    MenuItemRedo.Caption:='&Redo';
    if CanRedo
    then MenuItemRedo.Caption:=MenuItemRedo.Caption+' '+GetRedoName;
  end;
  if Assigned(MenuItemRepeat) then
  begin
    MenuItemRepeat.Enabled:=CanRepeat;
    MenuItemRepeat.Caption:='R&epeat';
    if CanRepeat
    then MenuItemRepeat.Caption:=MenuItemRepeat.Caption+' '+GetRepeatName;
  end;
end;


procedure TUndoAndRedo.LoadFromStream(Stream: TStream);
var
  t : integer;
  UARH: TUndoAndRedofileHeader;
  UARPH : TUndoAndRedoPointerRecord;
  UARNH : TUndoAndRedoNameRecord;
begin
  if not (Updating or InUndoMode or InRedoMode) then
  begin
    Reset;
    Stream.Read(UARH, SizeOf(UARH));
    NumberOfPointers:=UARH.NumberOfPoints;
    if (NumberOfPointers>0) and (UARH.Version<256*256) then
    begin
      CurrentPointer:=UARH.CurrentPointer;
      LastPointer:=UARH.LastPointer;
      NumberOfNames:=UARH.NumberOfNames;
      TheNames:=AllocMem(NumberOfNames);
      for t:=1 to NumberOfNames do
      begin
        Stream.Read(UARNH, SizeOf(UARNH));
        TheNames[t]:=UARNH.Name;
      end;
      MemoryAllocationSize:=0;
      ThePointers:=AllocMem(NumberOfPointers*SizeOf(TPointerRecord));
      for t:=1 to NumberOfPointers do
      begin
        Stream.Read(UARPH, SizeOf(UARPH));
        ThePointers[t].procedureVariable:=UARPH.procedureVariable;
        ThePointers[t].lengte1:=UARPH.Size1;
        ThePointers[t].lengte2:=UARPH.Size2;
        ThePointers[t].lengte3:=UARPH.Size3;
        ThePointers[t].Number:=UARPH.Number;
        ThePointers[t].procedureData1:=AllocMem(ThePointers[t].lengte1);
        ThePointers[t].procedureData2:=AllocMem(ThePointers[t].lengte2);
        ThePointers[t].procedureData3:=AllocMem(ThePointers[t].lengte3);
        Stream.Read(ThePointers[t].procedureData1^, ThePointers[t].lengte1);
        Stream.Read(ThePointers[t].procedureData2^, ThePointers[t].lengte2);
        Stream.Read(ThePointers[t].procedureData3^, ThePointers[t].lengte3);
        MemoryAllocationSize:=MemoryAllocationSize+ThePointers[t].lengte1+ThePointers[t].lengte2+ThePointers[t].lengte3+3*SizeOf(Longint)+SizeOf(Pointer);
      end;
    end;
    SetMenuItems;
  end;
end;


procedure TUndoAndRedo.SaveToStream(Stream: TStream);
var
  t : integer;
  UARH: TUndoAndRedofileHeader;
  UARPH : TUndoAndRedoPointerRecord;
  UARNH : TUndoAndRedoNameRecord;
begin
  if not (Updating or InUndoMode or InRedoMode) then
  begin
    UARH.Version:=256+8;
    UARH.VersionInfo[ 1]:='H';
    UARH.VersionInfo[ 2]:='i';
    UARH.VersionInfo[ 3]:='g';
    UARH.VersionInfo[ 4]:='h';
    UARH.VersionInfo[ 5]:='-';
    UARH.VersionInfo[ 6]:='L';
    UARH.VersionInfo[ 7]:='o';
    UARH.VersionInfo[ 8]:='g';
    UARH.VersionInfo[ 9]:='i';
    UARH.VersionInfo[10]:='c';
    UARH.NumberOfPoints:=NumberOfPointers;
    UARH.CurrentPointer:=CurrentPointer;
    UARH.LastPointer:=LastPointer;
    UARH.NumberOfNames:=NumberOfNames;
    UARH.Reserved:=0;
    Stream.Write(UARH, SizeOf(UARH));
    if (NumberOfPointers>0) then
    begin
      for t:=1 to NumberOfNames do
      begin
        UARNH.Name:=TheNames[t];
        Stream.Write(UARNH, SizeOf(UARNH));
      end;
      for t:=1 to NumberOfPointers do
      begin
        UARPH.procedureVariable:=ThePointers[t].procedureVariable;
        UARPH.Size1:=ThePointers[t].lengte1;
        UARPH.Size2:=ThePointers[t].lengte2;
        UARPH.Size3:=ThePointers[t].lengte3;
        UARPH.Number:=ThePointers[t].Number;
        Stream.Write(UARPH, SizeOf(UARPH));
        Stream.Write(ThePointers[t].procedureData1^, ThePointers[t].lengte1);
        Stream.Write(ThePointers[t].procedureData2^, ThePointers[t].lengte2);
        Stream.Write(ThePointers[t].procedureData3^, ThePointers[t].lengte3);
      end;
    end;
    SetMenuItems;
  end;
end;



procedure TUndoAndRedo.LoadFromFile(const Filename: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(Filename, fmOpenRead);
  try
    LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;


procedure TUndoAndRedo.SaveToFile(const Filename: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(Filename, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;



end.

